package com.z.toys.dns.provider;

import com.aliyun.alidns20150109.Client;
import com.aliyun.alidns20150109.models.*;
import com.aliyun.tea.TeaModel;
import com.z.toys.dns.model.ParamModel;
import com.z.toys.dns.util.StrUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Objects;

/**
 * @author wangzz
 * @since 2024/3/26
 **/
@Slf4j(topic = "AliDnsProvider")
public class AliDnsProvider implements DnsProvider {

    /**
     * 初使化账号
     * @return
     * @throws Exception
     * @author wangzz
     * @since 2024/3/26
     */
    public static Client createClient() throws Exception {
        String accessKeyId = System.getenv("ALIYUN_ACCESS_KEY_ID");
        String keySecret = System.getenv("ALIYUN_ACCESS_KEY_SECRET");
        if (StrUtil.isEmpty(accessKeyId) || StrUtil.isEmpty(keySecret)) {
            throw new RuntimeException("阿里云 accessKeyId 或 keySecret 未配置");
        }

        // Endpoint 规则为：[product_code].[regionid].aliyuncs.com
        // 请参考 https://api.aliyun.com/product/Alidns
        String endpoint = String.format("alidns.%s.aliyuncs.com", Objects.toString(System.getenv("ALIYUN_REGION_ID"), "cn-hangzhou"));
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config()
                .setAccessKeyId(accessKeyId)
                .setAccessKeySecret(keySecret)
                .setEndpoint(endpoint);

        return new Client(config);
    }

    @SneakyThrows
    public DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord getDomainInfo(ParamModel param) {

        Client client = createClient();
        DescribeDomainRecordsRequest req = new DescribeDomainRecordsRequest();
        req.setDomainName(param.getDomain());
        req.setRRKeyWord(param.getRr());

        log.debug("查询域名解析记录请求：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(req)));
        DescribeDomainRecordsResponse resp = client.describeDomainRecords(req);
        log.debug("查询域名解析记录返回：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp)));

        List<DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord> records = resp.getBody().getDomainRecords().getRecord();
        if (Objects.nonNull(records) && !records.isEmpty()) {
            return records.get(0);
        }

        return null;
    }

    @SneakyThrows
    @Override
    public void printDomainInfo(ParamModel param) {

        if (StrUtil.isEmpty(param.getDomain()) || StrUtil.isEmpty(param.getRr())) {
            log.error("必要参数不能为空");
            return;
        }

        DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord record = this.getDomainInfo(param);
        if (Objects.nonNull(record)) {
            log.info("查询域名结果↓");
            log.info("\t  recordId: {}", record.getRecordId());
            log.info("\tdomainName: {}", record.getDomainName());
            log.info("\t        rr: {}", record.getRR());
            log.info("\t      type: {}", record.getType());
            log.info("\t     value: {}", record.getValue());
            log.info("\t       ttl: {}", record.getTTL());
            log.info("\t    remark: {}", Objects.toString(record.getRemark(), ""));
        } else {
            log.info("域名未找到");
        }

    }

    @SneakyThrows
    @Override
    public void addDomain(ParamModel param) {

        if (StrUtil.isEmpty(param.getDomain()) || StrUtil.isEmpty(param.getRr()) || StrUtil.isEmpty(param.getType()) || StrUtil.isEmpty(param.getValue())) {
            log.error("必要参数不能为空");
            return;
        }

        DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord record = this.getDomainInfo(param);
        if (Objects.isNull(record)) {
            AddDomainRecordRequest req = new AddDomainRecordRequest();
            req.setDomainName(param.getDomain());
            req.setRR(param.getRr());
            req.setType(param.getType());
            req.setValue(param.getValue());
            log.debug("添加域名解析记录请求：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(req)));
            AddDomainRecordResponse resp = createClient().addDomainRecord(req);
            log.debug("添加域名解析记录返回：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp)));

            log.info("添加域名解析记录成功：recordId={}", resp.getBody().getRecordId());

        } else {
            log.info("域名记录已存在", param.getRr());
        }
    }

    @SneakyThrows
    @Override
    public void updateDomain(ParamModel param) {

        if (StrUtil.isEmpty(param.getDomain()) || StrUtil.isEmpty(param.getRr()) || StrUtil.isEmpty(param.getType()) || StrUtil.isEmpty(param.getValue())) {
            log.error("必要参数不能为空");
            return;
        }

        DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord record = this.getDomainInfo(param);
        if (Objects.nonNull(record)) {

            if (Objects.equals(param.getValue(), record.getValue())) {
                log.info("域名解析记录值相同，不更新");
                return;
            }

            UpdateDomainRecordRequest req = new UpdateDomainRecordRequest();
            req.setRecordId(record.getRecordId());
            req.setRR(param.getRr());
            req.setType(param.getType());
            req.setValue(param.getValue());
            log.debug("更新域名解析记录请求：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(req)));
            UpdateDomainRecordResponse resp = createClient().updateDomainRecord(req);
            log.debug("更新域名解析记录返回：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp)));

            log.info("更新域名解析记录成功");

        } else {
            log.info("域名未找到");
        }
    }

    @SneakyThrows
    @Override
    public void deleteDomain(ParamModel param) {

        if (StrUtil.isEmpty(param.getDomain()) || StrUtil.isEmpty(param.getRr())) {
            log.error("必要参数不能为空");
            return;
        }

        DescribeDomainRecordsResponseBody.DescribeDomainRecordsResponseBodyDomainRecordsRecord record = this.getDomainInfo(param);
        if (Objects.nonNull(record)) {

            DeleteDomainRecordRequest req = new DeleteDomainRecordRequest();
            req.setRecordId(record.getRecordId());
            log.debug("删除域名解析记录请求：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(req)));
            DeleteDomainRecordResponse resp = createClient().deleteDomainRecord(req);
            log.debug("删除域名解析记录返回：{}", com.aliyun.teautil.Common.toJSONString(TeaModel.buildMap(resp)));

            log.info("删除域名解析记录成功：recordId={}，domain={}，rr={}", record.getRecordId(), param.getDomain(), param.getRr());

        } else {
            log.info("域名未找到");
        }
    }
}
